#include <stdio.h>
#include <sys/time.h>
#include <sys/types.h>
#include <unistd.h>
#include <fcntl.h>
#include <termio.h>
#include <string.h>
#include <sys/socket.h>
#include <pthread.h>
#include <netinet/in.h>
#include <signal.h>
#include <syslog.h>

#include "RStest.h"

#define COM_MAX         4

typedef struct
{
  int                   fd;
  struct termios        oldtio;
  struct termios        newtio;
  unsigned char         buf;
} T_COMPORT;


static char           * coms[COM_MAX]  = {"/dev/ttyS0","/dev/ttyS1","/dev/ttyS2","/dev/ttyS3"};
static T_COMPORT      * ports[COM_MAX] = {NULL,NULL,NULL,NULL};

int com_Init ( unsigned char Port, struct ser_conf *ConfigStruct)
{
  if( !ports[Port] )
  if( (ports[Port] = malloc( sizeof(T_COMPORT) )) != NULL )
  {
    ports[Port]->fd = open( coms[Port], O_RDWR | O_NOCTTY | O_EXCL | O_NONBLOCK );
    //
    if( ports[Port]->fd > 0 )
    {
      tcgetattr( ports[Port]->fd, &ports[Port]->oldtio );
      bzero( &ports[Port]->newtio, sizeof(struct termios) );
      ports[Port]->newtio.c_cflag = ConfigStruct->SerCFlags | CLOCAL | CREAD;
      ports[Port]->newtio.c_iflag = ConfigStruct->SerIFlags | IGNPAR;
      ports[Port]->newtio.c_oflag = 0;
      ports[Port]->newtio.c_lflag = 0;
      ports[Port]->newtio.c_cc[VTIME] = 5;
      ports[Port]->newtio.c_cc[VMIN]  = 1;
      tcflush( ports[Port]->fd, TCIFLUSH );
      tcsetattr( ports[Port]->fd, TCSANOW, &ports[Port]->newtio );
      fprintf(stderr,"COM Init OK %d %s\n", ports[Port]->fd, coms[Port]);

      return 1;
    }
  }
		fprintf(stderr,"COM Init Err %s\n", coms[Port]);
  return 0;
}


int com_Write(unsigned char Port, unsigned char *bytes, int numbytes)
{
	int i, rc, sfd;
	
	sfd = ports[Port]->fd;
	

	(void) tcflush(sfd, TCIFLUSH);
	(void) tcflush(sfd, TCOFLUSH);

	if(debug) {
		fprintf(stderr, "SEND(fd=%d, %d):", sfd, numbytes);

		for(i=0; i<numbytes; i++)	
			fprintf(stderr," 0x%02X", (unsigned char) bytes[i]);

		fprintf(stderr,"\n");
	}
	

	i = write(sfd, bytes, numbytes);

	
	if(i > 0) {
		rc = tcdrain(sfd);
		if(debug && rc!=0) {
			perror("tcdrain()");
		}
	} else {
		if(debug)
			perror("write()");
	}
	
	return i;
}

int com_Read(unsigned char Port, unsigned char *bytes, int numbytes, int mstimeout,
				int msgap)
{
	fd_set fds;
	struct timeval timeout, now, end;
	long gsec, gusec;
	long tsec, tusec;
	int count = 0;
	int rc, sfd;
	
	sfd = ports[Port]->fd;
	
	/* Calculate gap time values */
	gsec = msgap/1000;
	gusec = (msgap%1000) * 1000;

	/* calculate timeout time values */
	tsec = mstimeout/1000;
	tusec = (mstimeout%1000) * 1000;

	/*
	 * Get the current time of day
	 */
	if(gettimeofday(&now, NULL)!=0) {
		perror("gettimeofday()");
		return -1;
	}


	/*
	 * Figure out the time at which we will
	 * have timed out
	 */
	end.tv_sec = now.tv_sec + tsec;
	end.tv_usec = now.tv_usec + tusec;
	if(end.tv_usec > 1000000) {
		end.tv_usec -= 1000000;
		end.tv_sec += 1;
	}

	FD_ZERO(&fds);
	FD_SET(sfd, &fds);
	timeout.tv_sec = tsec;
	timeout.tv_usec = tusec;

	rc = select(sfd+1, &fds, NULL, NULL, &timeout);

	if(debug && rc==0) {
		fprintf(stderr, "timeout\n");
	}
	if(rc==0 || rc==-1) {
		return rc;
	}
	

	count = read(sfd, bytes, numbytes);

	
	if(count<0) {
		perror("read()");
	}


	if(count==numbytes) {
		return count;
	}


	while(1) {

		FD_SET(sfd, &fds);
		timeout.tv_sec = gsec;
		timeout.tv_usec = gusec;
		rc = select(sfd+1, &fds, NULL, NULL, &timeout);

		if(rc==-1) {
			if(debug) {
				perror("select()");
			}
			return -1;
		} else if(rc==0) {
			return count;
		}
		
		rc = read(sfd, bytes+count, numbytes-count);
		if(rc==-1) {
			if(debug) {
				perror("read()");
			}
			return -1;
		} else if(rc==0) {
			if(debug) {
				fprintf(stderr, "Warning: read() returned 0 after select\n");
			}
			return count;
		} else {
			
			count += rc;
			if(count==numbytes) {
				return count;
			}

			/* find out what time it is now */
			if(gettimeofday(&now, NULL)!=0) {
				perror("gettimeofday()");
				return -1;
			}

			if((now.tv_sec > end.tv_sec)
			   || (now.tv_sec>=end.tv_sec
			       && (now.tv_usec >= end.tv_usec)))
				return count;
		}
	} 

}

	
void     com_Uninit  ( unsigned char Port )
{
  if( ports[Port] )
  {
    tcsetattr( ports[Port]->fd, TCSANOW, &ports[Port]->oldtio );
    close( ports[Port]->fd );
    free( ports[Port] );
    ports[Port] = NULL;
  }
}

int COM_read_thread(unsigned char Port)
{
		sigset_t sigs;
		int count;

		sigemptyset(&sigs);
		sigaddset(&sigs, SIGINT);
		sigaddset(&sigs, SIGTERM);
		
		if(pthread_sigmask(SIG_BLOCK, &sigs, NULL) != 0) {
		if(debug) {
			fprintf(stderr, "pthread_sigmask() failed\n");
		}
		}
		
		count = com_Read(Port, response, BUFFERSIZE, 1000, 800);

		return count;
}

int COM_write_thread(unsigned char Port)
{
		sigset_t sigs;
		int count,i;

		sigemptyset(&sigs);
		sigaddset(&sigs, SIGINT);
		sigaddset(&sigs, SIGTERM);
		
		if(pthread_sigmask(SIG_BLOCK, &sigs, NULL) != 0) {
		if(debug) {
			fprintf(stderr, "pthread_sigmask() failed\n");
		}
		}
		
		srand(time(NULL));
	
		for(i =0; i< BUFFERSIZE; i++){
		packet[i] = rand() % (RANGE+1);
		}
		
		count = com_Write(Port, packet, BUFFERSIZE);

		return count;
}
