
#include <stdio.h>
#include <stdarg.h>
#include <memory.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/mman.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <signal.h>
#include <unistd.h>

#include "libnpe.h"

#define PIOA_OFFSET     0x400
#define PIOB_OFFSET     0x600
#define PIOC_OFFSET     0x800
#define PIOD_OFFSET     0xa00
#define PMC_OFFSET      0xc00

#define PIOA_ID         2
#define PIOB_ID         3
#define PIOC_ID         4
#define PIOD_ID         5

/* Memory mapping definitions */
#define MAP_SIZE 4096UL
#define MAP_MASK (MAP_SIZE - 1)

/* AT91RM9200 PIO definitions */
#define AT91_SYS        0xFFFFF000
//#define AT91_PIOA       0xFFFFF400
//#define AT91_PIOB       0xFFFFF600
//#define AT91_PIOC       0xFFFFF800
//#define AT91_PIOD       0xFFFFFA00

#define PMC_PCER        0x0010

#define PIO_PER         0x0000
#define PIO_PDR         0x0004
#define PIO_OER         0x0010
#define PIO_ODR         0x0014
#define PIO_SODR        0x0030
#define PIO_CODR        0x0034
#define PIO_PDSR        0x003c

const int       port_id_tab[PORT_MAX] =
{
  PIOA_ID,
  PIOB_ID,
  PIOC_ID,
  PIOD_ID
};

const int       port_addr_tab[PORT_MAX] =
{
  PIOA_OFFSET,
  PIOB_OFFSET,
  PIOC_OFFSET,
  PIOD_OFFSET
};

//----------------------------------------------------------------
// Function :   port_pin_operation
// Notes    :
// History  :
//----------------------------------------------------------------

int port_pin_operation( T_PORT port, int pin, char op )
{
        int fd, addr, val = 0, ret = 0;
        void * base;

        if((fd = open("/dev/mem", O_RDWR | O_SYNC)) == -1){
                DebugMSG(DEBUG_ERR, "gpio: Error opening /dev/mem\n");
                return -1;
        }
        base = mmap(0, MAP_SIZE, PROT_READ | PROT_WRITE, MAP_SHARED, fd, AT91_SYS & ~MAP_MASK);
        if(base == (void *) -1){
                DebugMSG(DEBUG_ERR, "gpio: Error mapping memory\n");
                close(fd);
                return -1;
        }

        addr = port_addr_tab[port];

        if( op != '?' && op != '=' )
        {
            val = 1 << pin;
            *((unsigned long*)(base + ((addr + PIO_PER) & MAP_MASK))) = val;
            *((unsigned long*)(base + ((addr + PIO_OER) & MAP_MASK))) = val;
        }
        switch( op )
        {
            case '+':
                *((unsigned long*)(base + ((addr + PIO_SODR) & MAP_MASK))) = val;
            break;
            case '-':
                *((unsigned long*)(base + ((addr + PIO_CODR) & MAP_MASK))) = val;
            break;
            case '?':
                /* First, enable PIO clock */
                val = 1 << port_id_tab[port];
                *((unsigned long*)(base + ((PMC_OFFSET + PMC_PCER) & MAP_MASK))) = val;
                /* Then read port value */
                val = *((unsigned long*)(base + ((addr + PIO_PDSR) & MAP_MASK)));
                ret = (val >> pin) & 0x00000001;
            break;
            case '=':
                /* First, enable PIO clock */
                val = 1 << port_id_tab[port];
                *((unsigned long*)(base + ((PMC_OFFSET + PMC_PCER) & MAP_MASK))) = val;
                /* Then read port value */
                ret = *((unsigned long*)(base + ((addr + PIO_PDSR) & MAP_MASK)));
            break;
            default:
            break;
        };
        if(munmap(base, MAP_SIZE) == -1){
                DebugMSG(DEBUG_ERR, "Error unmapping memory\n");
                close(fd);
                return -1;
        }
        close(fd);
        return ret;
}


