
#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <memory.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/mman.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <signal.h>
#include <ctype.h>
#include <unistd.h>

#include "libnpe.h"

/* Memory mapping definitions */
#define MAP_SIZE 256UL
#define MAP_MASK (MAP_SIZE - 1)

/* AT91RM9200 PIO definitions */
#define AT91_SYS        0x50000000

//----------------------------------------------------------------
// Function :   io_port_write
// Notes    :
// History  :
//----------------------------------------------------------------

int io_port_write( int port, int val )
{
        int fd;
        void * base;

        if((fd = open("/dev/mem", O_RDWR | O_SYNC)) == -1){
                DebugMSG(DEBUG_ERR, "io: Error opening /dev/mem\n");
                return -1;
        }

        /* Map one page */
        base = mmap(0, MAP_SIZE, PROT_READ | PROT_WRITE, MAP_SHARED, fd, AT91_SYS & ~MAP_MASK);
        if(base == (void *) -1){
                DebugMSG(DEBUG_ERR, "io: Error mapping memory\n");
                close(fd);
                return -1;
        }

        *((unsigned long*)(base + port)) = (val&0xFF) << 16;

        /* Unmap previously mapped page */
        if(munmap(base, MAP_SIZE) == -1){
                DebugMSG(DEBUG_ERR, "io: Error unmapping memory\n");
                close(fd);
                return -1;
        }

        close(fd);
        return 0;
}

//----------------------------------------------------------------
// Function :   io_port_read
// Notes    :
// History  :
//----------------------------------------------------------------

int io_port_read( int port )
{
        int fd, val = -1;
        void * base;

        if((fd = open("/dev/mem", O_RDWR | O_SYNC)) == -1){
                DebugMSG(DEBUG_ERR, "io: Error opening /dev/mem\n");
                return -1;
        }

        /* Map one page */
        base = mmap(0, MAP_SIZE, PROT_READ | PROT_WRITE, MAP_SHARED, fd, AT91_SYS & ~MAP_MASK);
        if(base == (void *) -1){
                DebugMSG(DEBUG_ERR, "io: Error mapping memory\n");
                close(fd);
                return -1;
        }

        val = *((unsigned long*)(base + port));
        val = (val&0xF0000) >> 16;

        /* Unmap previously mapped page */
        if(munmap(base, MAP_SIZE) == -1){
                DebugMSG(DEBUG_ERR, "io: Error unmapping memory\n");
                close(fd);
                return -1;
        }

        close(fd);
        return val;
}
