
#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <memory.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/mman.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <signal.h>
#include <ctype.h>
#include <unistd.h>
#include <time.h>
#include <linux/i2c.h>
#include <bcd.h>

#include "libnpe.h"

typedef struct
{
  char		hsec;
  char		sec;
  char		min;
  char		hour;
  char		year;
  char		mth;
} t_pcf8583_time;

int pcf8583_open( char addr )
{
        int adr = (addr>>1);
	int fd;
	
    	if((fd = open("/dev/i2c-0", O_RDWR)) < 0){
		DebugMSG(DEBUG_ERR, "i2c: Error opening /dev/i2c\n");
		return -1;
	}
//	DebugMSG(DEBUG_ERR, "i2c: device open\n");
	
	if( ioctl(fd,I2C_SLAVE,adr) < 0 ){
		DebugMSG(DEBUG_ERR, "i2c: Error setting i2c\n");
		close(fd);
		return -1;
	}
//	DebugMSG(BEBUG_ERR, "i2c: device set\n");
	return fd;
}

int pcf8583_close( int fd )
{
	close(fd);
//	DebugMSG(DEBUG_ERR, "i2c: device closed\n");
	return 0;
}

int pcf8583_reg_read(int fd, unsigned char reg, char * val)
{
	unsigned char buf[1] = {reg};
	
	if( write(fd,buf,1) != 1 ){
		DebugMSG(DEBUG_ERR, "i2c: Write error\n");
		return -1;
	}
	if( read(fd,buf,1) != 1 ){
		DebugMSG(DEBUG_ERR, "i2c: Read error\n");
		return -1;
	}
	*val = buf[0];
	return 0;
}

int pcf8583_reg_write(int fd, unsigned char reg, char val)
{
	unsigned char buf[2] = {reg,val};
	
	if( write(fd,buf,2) != 2 ){
		DebugMSG(DEBUG_ERR, "i2c: Write error\n");
		return -1;
	}
	return 0;
}

int pcf8583_buf_read(int fd, unsigned char reg, char * buf, int len)
{
	if( (reg+len) > PCF8583_MAX_OFFS )	return -1;

	while( len-- )
	{
	    if( pcf8583_reg_read(fd, reg, buf) < 0 ){
		    return -1;
	    }
	    reg++;
	    buf++;
	}
	return 0;
}

int pcf8583_buf_write(int fd, unsigned char reg, char * buf, int len)
{
	if( (reg+len) > PCF8583_MAX_OFFS )	return -1;

	while( len-- )
	{
	    if( pcf8583_reg_write(fd, reg, *buf) < 0 ){
		    return -1;
	    }
	    reg++;
	    buf++;
	}
	return 0;
}

int pcf8583_get_time( int fd, struct tm * tm )
{
    t_pcf8583_time	rtc;
    
    if( pcf8583_buf_read(fd, PCF8583_RTC_OFFS, (char*)&rtc, sizeof(rtc)) < 0 ){
	    return -1;
    }
    tm->tm_sec  = BCD2BIN( rtc.sec );
    tm->tm_min  = BCD2BIN( rtc.min );
    tm->tm_hour = BCD2BIN( rtc.hour & 0x3F );
    tm->tm_mday = BCD2BIN( rtc.year & 0x3F );
    tm->tm_year = (rtc.year >> 6) + 104;
    tm->tm_mon  = BCD2BIN( rtc.mth & 0x1F ) - 1;
    tm->tm_wday = rtc.mth >> 5;
    
    mktime(tm);
    
//    DebugMSG(DEBUG_STATUS, "Time: %s\n", asctime(tm));
    
    return 0;
}

int pcf8583_set_time( int fd, struct tm * tm )
{
    t_pcf8583_time	rtc;
    
    mktime(tm);
    
    rtc.sec   = BIN2BCD( tm->tm_sec );
    rtc.min   = BIN2BCD( tm->tm_min );
    rtc.hour  = BIN2BCD( tm->tm_hour );
    rtc.year  = BIN2BCD( tm->tm_mday );
    rtc.year |= (tm->tm_year%4) << 6;
    rtc.mth   = BIN2BCD( tm->tm_mon ) + 1;
    rtc.mth  |= tm->tm_wday << 5;
    
//    DebugMSG(DEBUG_STATUS, "Time: %s\n", asctime(tm));
    
    if( pcf8583_buf_write(fd, PCF8583_RTC_OFFS, (char*)&rtc, sizeof(rtc)) < 0 ){
	    return -1;
    }
    return 0;
}

