
#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <memory.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/mman.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <signal.h>
#include <ctype.h>
#include <unistd.h>
#include <termio.h>

#include "libnpe.h"

#define COM_MAX         4

typedef struct
{
  int                   fd;
  struct termios        oldtio;
  struct termios        newtio;
  unsigned char         buf;
} T_COMPORT;

static char           * coms[COM_MAX]  = {"/dev/ttyS0","/dev/ttyS1","/dev/ttyS2","/dev/ttyS3"};
static T_COMPORT      * ports[COM_MAX] = {NULL,NULL,NULL,NULL};

//----------------------------------------------------------------
// Function :   com_Init
// Notes    :
// History  :
//----------------------------------------------------------------

int     com_Init ( unsigned char Port, unsigned long BRate, unsigned char Len, unsigned char Par, unsigned char Stop )
{
  if( !ports[Port] )
  if( (ports[Port] = malloc( sizeof(T_COMPORT) )) != NULL )
  {
    ports[Port]->fd = open( coms[Port], O_RDWR | O_NOCTTY | O_EXCL | O_NONBLOCK );
    //
    if( ports[Port]->fd > 0 )
    {
      tcgetattr( ports[Port]->fd, &ports[Port]->oldtio );
      bzero( &ports[Port]->newtio, sizeof(struct termios) );
      ports[Port]->newtio.c_cflag = BRate | CRTSCTS | CS8 | CLOCAL | CREAD;
      ports[Port]->newtio.c_iflag = IGNPAR;
      ports[Port]->newtio.c_oflag = 0;
      ports[Port]->newtio.c_lflag = 0;
      ports[Port]->newtio.c_cc[VTIME] = 5;
      ports[Port]->newtio.c_cc[VMIN]  = 1;
      tcflush( ports[Port]->fd, TCIFLUSH );
      tcsetattr( ports[Port]->fd, TCSANOW, &ports[Port]->newtio );

      DebugMSG(DEBUG_STATUS,"\nCOM Init OK %d %s", ports[Port]->fd, coms[Port]);

      return 1;
    }
  }
  DebugMSG(DEBUG_STATUS,"\nCOM Init Err %s", coms[Port]);

  return 0;
}

//----------------------------------------------------------------
// Function :   com_RxStat
// Notes    :
// History  :
//----------------------------------------------------------------

short int   com_RxStat  ( unsigned char Port )
{
  if( ports[Port] )
  if( read( ports[Port]->fd, &ports[Port]->buf, 1 ) == 1 )
  {
    return 1;
  }
  return 0;
}

//----------------------------------------------------------------
// Function :   com_GetChar
// Notes    :
// History  :
//----------------------------------------------------------------

unsigned char    com_GetChar ( unsigned char Port )
{
  return ports[Port]->buf;
}

//----------------------------------------------------------------
// Function :   com_Flush
// Notes    :
// History  :
//----------------------------------------------------------------

void     com_Flush   ( unsigned char Port )
{
  if( ports[Port] )
  {
    tcflush( ports[Port]->fd, TCIFLUSH );
    tcflush( ports[Port]->fd, TCOFLUSH );
  }
}

//----------------------------------------------------------------
// Function :   com_Uninit
// Notes    :
// History  :
//----------------------------------------------------------------

void     com_Uninit  ( unsigned char Port )
{
  if( ports[Port] )
  {
    tcsetattr( ports[Port]->fd, TCSANOW, &ports[Port]->oldtio );
    close( ports[Port]->fd );
    free( ports[Port] );
    ports[Port] = NULL;
  }
}

//----------------------------------------------------------------
// Function :   com_PutChar
// Notes    :
// History  :
//----------------------------------------------------------------

int     com_PutChar ( unsigned char Port, unsigned char c )
{
  if( ports[Port] )
  if( write( ports[Port]->fd, &c, 1 ) == 1 )
  {
    return 1;
  }
  return 0;
}

//----------------------------------------------------------------
// Function :   com_TxWait
// Notes    :
// History  :
//----------------------------------------------------------------

void     com_TxWait  ( unsigned char Port )
{
  if( ports[Port] )
  {
    tcdrain( ports[Port]->fd );
  }
}

//
