
#include <stdio.h>
#include <stdarg.h>
#include <memory.h>
#include <errno.h>
#include <stdlib.h>
#include <ctype.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/mman.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <signal.h>
#include <time.h>

#include "libnpe.h"

//----------------------------------------------------------------
// Function :   main
// Notes    :
// History  :
//----------------------------------------------------------------

int main(int argc, char *argv[])
{
        T_PORT id = PORT_PIOA;
        int i, rc, adr = 0, pin = 0, retval = -1, val = 0;
        char *current, action, port;


        if(argc < 2){
                DebugMSG(DEBUG_ERR, "NPE user-space control utility, MMSoft (c)2007\n");
                DebugMSG(DEBUG_ERR, "Usage: npe action action ... action\n");		
                DebugMSG(DEBUG_ERR, "Where action:\n");
                DebugMSG(DEBUG_ERR, "  T[S|R] - NV RTC control\n");
                DebugMSG(DEBUG_ERR, "           S - save current time to NV RTC\n");
                DebugMSG(DEBUG_ERR, "           R - restore time from NV RTC\n");
                DebugMSG(DEBUG_ERR, "  [+|-|?]P[A|B|C|D][pin] - processor port pin control\n");
                DebugMSG(DEBUG_ERR, "           + - set port pin value to 1\n");
                DebugMSG(DEBUG_ERR, "           - - clear port pin value to 0\n");
                DebugMSG(DEBUG_ERR, "           ? - read port pin value (0/1)\n");
                DebugMSG(DEBUG_ERR, "           A,B,C,D - processor port ID\n");
                DebugMSG(DEBUG_ERR, "           pin - port pin number (0-31)\n");
                DebugMSG(DEBUG_ERR, "  [=]PV[adr]=[val] - NV Memory control\n");
                DebugMSG(DEBUG_ERR, "           = - write NV memory NV[adr]=val\n");
                DebugMSG(DEBUG_ERR, "  [?]PV[adr] - NV Memory control\n");
                DebugMSG(DEBUG_ERR, "           ? - read NV memory NV[adr]\n");
                DebugMSG(DEBUG_ERR, "  [?]PI[adr] - Input port control\n");
                DebugMSG(DEBUG_ERR, "           ? - read Input port IN[adr]\n");
                DebugMSG(DEBUG_ERR, "  [+|-]PO[adr]=[pin] - Output port pin control\n");
                DebugMSG(DEBUG_ERR, "           + - set Out port pin value to 1\n");
                DebugMSG(DEBUG_ERR, "           - - clear Out port pin value to 0\n");
                DebugMSG(DEBUG_ERR, "           pin - port pin number (0-7)\n");
                DebugMSG(DEBUG_ERR, "  [=]PO[adr]=[val] - Output port control\n");
                DebugMSG(DEBUG_ERR, "           = - set Out port value OUT[adr]=val\n");
                DebugMSG(DEBUG_ERR, "  [?|*]PO[adr] - Output port control\n");
                DebugMSG(DEBUG_ERR, "           ? - read buffered Out port value OUT[adr]\n");
                DebugMSG(DEBUG_ERR, "           * - refresh Out port value OUT[adr]\n");
                DebugMSG(DEBUG_ERR, "Note: supported multiple [+|-|=] and only one [?] request per each call\n");
                DebugMSG(DEBUG_ERR, "      time control action [T] can not be used with other requests\n");
                DebugMSG(DEBUG_ERR, "      [adr|val] are in HEX format, [pin] are in DEC format\n");
                exit(0);
        }

	if(tolower(argv[1][0]) == 't')
	{
		struct tm tm;
		time_t t;
		int i = pcf8583_open( PCF8583_ADDR0 );		
				
		if(tolower(argv[1][1]) == 'r')
		{
		    pcf8583_get_time( i, &tm );
		
		    t = mktime(&tm);
		    stime(&t);

		    DebugMSG(DEBUG_ERR, "Time restored: %s", ctime(&t) );		    
		} 
		else if(tolower(argv[1][1]) == 's')
		{
		    t = time(NULL);
		    pcf8583_set_time( i, localtime( &t ) );

		    DebugMSG(DEBUG_ERR, "Time saved: %s", ctime(&t) );
		}
		
		pcf8583_close( i );
		exit(0);
	}
	
        for(i = 1; i < argc; i++){ /* Parse all given arguments */
                rc = 0;
                current = argv[i];
                action = current[0];

                if(tolower(current[1]) != 'p') rc = -1;

                port = tolower(current[2]);

		if( port == 'v' ){
            	    if(action == '='){
			sscanf( &current[3], "%X=%X", &adr, &val );		    
			if( adr < 1 || adr > 239 ) rc = -1;
			if( val < 0 || val > 0xFF ) rc = -1;
		    } else if( action == '?'){
			sscanf( &current[3], "%X", &adr );		    
			if( adr < 1 || adr > 239 ) rc = -1;
		    } else rc = -1;
		} else if( port == 'i' ){
            	    if(action == '?'){
			sscanf( &current[3], "%X", &adr );		    
			adr <<= 2;
			if( adr < 0 || adr > 255 ) rc = -1;
		    } else rc = -1;
		} else if( port == 'o' ){
            	    if(action == '+' || action == '-'){
			sscanf( &current[3], "%X=%d", &adr, &pin );		    
			adr <<= 2;
			if( adr < 0 || adr > 255 ) rc = -1;
			if( pin < 0 || pin > 7 ) rc = -1;
		    } else if( action == '='){
			sscanf( &current[3], "%X=%X", &adr, &val );		    
			adr <<= 2;
			if( adr < 0 || adr > 255 ) rc = -1;
			if( val < 0 || val > 0xFF ) rc = -1;
		    } else if( action == '?' || action == '*'){
			sscanf( &current[3], "%X", &adr );		    
			adr <<= 2;
			if( adr < 0 || adr > 255 ) rc = -1;
		    } else rc = -1;
		} else if(port >= 'a' && port <= 'd'){
            	    if(action == '+' || action == '-' || action == '?'){
			sscanf( &current[3], "%d", &pin );
			if( pin < 0 || pin > 31 ) rc = -1;
		    } else rc = -1;
		} else rc = -1;

                if(rc == -1){
                        DebugMSG(DEBUG_ERR, "testio: Error in given argument '%s', skipping\n", current);
                        exit(-1);
                }
		retval = -1;
                switch(port){
                        case 'd':
			    id++;
                        case 'c':
			    id++;
                        case 'b':
			    id++;
                        case 'a':		
            		    retval = port_pin_operation( id, pin, action );
			break;
                        case 'o':
			  {
			    int fd = pcf8583_open( PCF8583_ADDR0 );		
			    char r;
			    if( pcf8583_reg_read( fd, 16, &r ) == 0 )
			    {
				switch(action)
				{
				    case '+':
				        r |= (1<<pin);
					retval = io_port_write( adr, r );
				    break;
				    case '-':
				        r &= ~(1<<pin);
					retval = io_port_write( adr, r );
				    break;
				    case '=':
				        r = val;
					retval = io_port_write( adr, r );
				    break;
				    case '*':
					retval = io_port_write( adr, r );
				    break;
				    case '?':
					retval = r;
				    break;
				    default:
				    break;
				};
				pcf8583_reg_write( fd, 16, r );
			    }
			    pcf8583_close( fd );
			  }
			break;
                        case 'i':
			    switch(action)
			    {
    				case '?':
				    retval = io_port_read( adr );
				break;
				default:
				break;
			    };
			break;
                        case 'v':
			  {
			    int fd = pcf8583_open( PCF8583_ADDR0 );		
			    char r;
			    switch(action)
			    {
				case '=':
				    retval = pcf8583_reg_write( fd, adr+16, val );
				break;
				case '?':
				    if( pcf8583_reg_read( fd, adr+16, &r ) == 0 )
				    {
				      retval = r;
				    }
				break;
				default:
				break;
			    };
			    pcf8583_close( fd );
			  }
			break;
			default:
			break;
                }
		
		if( action == '?' ){
//                    DebugMSG(DEBUG_ERR, "%X\n",retval);
		    exit(retval);
		}		
		if( retval < 0 ){
                    DebugMSG(DEBUG_ERR, "npe: Error in IO operation\n");
		    exit(-1);
		}
        }
        exit(retval);
}
