/*
 *	Description: 
 *		tcp client example
 *		tcp echo client: echo back the received data.
 *
 *		What is shown in this example:
 *		1. How to program a tcp client
 *
 *	Usage:  ./tcp_client.o server_IP server_port
 *		
 */
 

#include <stdio.h>
#include <memory.h>
#include <errno.h>
#include <stdlib.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <fcntl.h>
#include <signal.h>
#include <time.h>
#include <unistd.h>
#include <arpa/inet.h>

#include "libnpe.h"

int main(int argc, char *argv[])
{
	int	fd, tcp_port, i, ret;
	struct sockaddr_in dest;
	char	buf[1024];

	if ( argc != 3 ) {
		DebugMSG(DEBUG_ERR, "Usage: ./tcp_client server_IP server_port\n");
        	exit(-1);
        } 

	/*create a socket*/
	fd = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
	if (fd < 0) {
  		DebugMSG(DEBUG_ERR, "socket failed\n");
  		exit(0);		
	}
	
	bzero(&dest, sizeof(dest));
	dest.sin_family = AF_INET;
	tcp_port = atoi(argv[2]);	/*get server port*/
	dest.sin_port = htons(tcp_port);
	inet_aton(argv[1], &dest.sin_addr); /*get server ip*/

	/*connect to server, retry 3 times*/
	for(i=0; i<3; i++) {
		ret = connect(fd, (struct sockaddr*)&dest, sizeof(dest));
		if (ret == 0) {
			DebugMSG(DEBUG_ERR,"connect ok\r\n");
			break;
		}
	}
	if (i == 3) {
		DebugMSG(DEBUG_ERR,"connect failed\r\n");
		close(fd);
		exit(0);
	}

	for(;;) {
		/*when data received, send them back*/
		ret = recv(fd, buf, sizeof(buf), 0);
		if (ret > 0) {
			send(fd, buf, ret, 0);
			buf[ret] = 0;
			DebugMSG(DEBUG_ERR, "Received: %s \n\n", buf);
		} else if (ret == 0) {
			DebugMSG(DEBUG_ERR, "disconnect by remote\r\n");
			break;
		} else if (ret < 0) {
			DebugMSG(DEBUG_ERR, "read from ethernet failed, errno: %d\r\n", errno);
			break;
		}
	}
	close(fd);
	exit(0);
}
