/*
 *	Description: 
 *		tcp server example
 *		tcp echo server: listen tcp port 1000, and echo back the received data.
 *
 *		What is shown in this example:
 *		1. How to program a tcp echo server
 *		
 */
#include <stdio.h>
#include <memory.h>
#include <errno.h>
#include <stdlib.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <fcntl.h>
#include <signal.h>
#include <time.h>
#include <unistd.h>
#include <arpa/inet.h>

#include "libnpe.h"

int main(int argc, char *argv[])
{
	int	fd_listen, fd_client, size, ret, bi_er, tcp_port;
	struct sockaddr_in dest;
	char	buf[1024];

	if ( argc != 2 ) {
		DebugMSG(DEBUG_ERR, "Usage: ./tcp_server server_port\n");
        	exit(-1);
        } 

	/*create a socket*/
	fd_listen = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
	if (fd_listen < 0) {
  		DebugMSG(DEBUG_ERR,"socket failed\n");
  		exit(0);		
	}

	/*use bind function to associate an address with a socket*/
	bzero(&dest, sizeof(dest));
	dest.sin_family = AF_INET;
	tcp_port = atoi(argv[1]);
	dest.sin_port = htons(tcp_port);
	dest.sin_addr.s_addr = htonl(INADDR_ANY);
	bi_er = bind(fd_listen, (struct sockaddr*)&dest, sizeof(dest));
	if(bi_er == -1) {
		DebugMSG(DEBUG_ERR,"bind error \n");
		close(fd_listen);
		exit(0);
	}

	/*listen 1 connect request*/
	DebugMSG(DEBUG_ERR,"Waiting for request \n");
	listen(fd_listen, 1);

	size = sizeof(dest);
	/*retrieve a connect request*/
       	fd_client = accept(fd_listen, (struct sockaddr*)&dest, (socklen_t *)&size);
	if (fd_client == -1) {
		DebugMSG(DEBUG_ERR,"accept failed, errno: %d\r\n", errno);
		close(fd_listen);
		exit(0);
       	}
		DebugMSG(DEBUG_ERR,"accept: %s, port %d\r\n", 
			inet_ntoa(dest.sin_addr), htons(dest.sin_port));

	for(;;) {
		/*when data received, send them back*/
		ret = recv(fd_client, buf, sizeof(buf), 0);
		if (ret > 0) {
			send(fd_client, buf, ret, 0);
			buf[ret] = 0;
			DebugMSG(DEBUG_ERR, "Received: %s \n\n", buf);
		} else if (ret == 0) {
			DebugMSG(DEBUG_ERR,"disconnect by remote\r\n");
			break;
		} else if (ret < 0) {
			DebugMSG(DEBUG_ERR,"read from ethernet failed, errno: %d\r\n", errno);
			break;
		}
	}
	close(fd_listen);
	close(fd_client);
	exit(0);
}
